import { Context, DownloadOptions, HTTPClient, TransferProgress } from "koajax";
import { FileSystemHandle } from "native-file-system-adapter";
import { ReadableStream } from "web-streams-polyfill";
import { ByteSize, Constructor } from "web-utility";
export abstract class DownloadTask implements Partial<TransferProgress> {
    abstract client: Pick<HTTPClient<Context>, 'download'>;
    accessor id: string;
    accessor name: string;
    accessor path: string;
    accessor fsHandle: FileSystemHandle | undefined;
    accessor total: number;
    accessor loaded: number;
    accessor percent: number;
    accessor executing: boolean;
    accessor options: DownloadOptions | undefined;
    stream?: ReadableStream<Partial<TransferProgress>>;
    get totalSize(): ByteSize;
    get loadedSize(): ByteSize;
    static nameOf(URI: string): string;
    constructor(path: string, name?: string);
    toJSON(): {
        total: number;
        loaded: number;
        percent: number;
        id: string;
        name: string;
        path: string;
        fsHandle: FileSystemHandle;
        options: DownloadOptions;
    };
    saveMeta(data?: Partial<TransferProgress>): Partial<TransferProgress>;
    abstract loadStream(options?: DownloadOptions): AsyncGenerator<Partial<TransferProgress>>;
    start(options?: DownloadOptions): Promise<ReadableStream<Partial<TransferProgress>>>;
    pause(): Promise<void>;
    destroy(): Promise<void>;
}
export class HTTPDownloadTask extends DownloadTask {
    client: DownloadTask['client'];
    constructor(path: string, name?: string);
    loadStream(options?: DownloadOptions): AsyncGenerator<Partial<import("koajax").TransferProgress>, void, unknown>;
}
export class Downloader {
    static protocolMap: Record<string, Constructor<DownloadTask>>;
    static createTask(path: string, name?: string): DownloadTask;
    constructor();
    accessor tasks: DownloadTask[];
    get unfinishedCount(): number;
    get executingCount(): number;
    createTask(path: string, name?: string): DownloadTask;
    destroyTask(name: string): Promise<void>;
}

//# sourceMappingURL=index.d.ts.map
